from .chunk import Chunk
from .knowledge import (
    EmbeddingModelEnum,
    Knowledge,
    KnowledgeCreate,
    KnowledgeSourceEnum,
    KnowledgeSplitConfig,
    KnowledgeTypeEnum,
)
from .page import PageParams, PageResponse
from .retrieval import (
    RetrievalByKnowledgeRequest,
    RetrievalBySpaceRequest,
    RetrievalChunk,
    RetrievalEnum,
)
from .task import Task, TaskStatus
from .tenant import Tenant

__all__ = [
    "Chunk",
    "KnowledgeSourceEnum",
    "KnowledgeTypeEnum",
    "EmbeddingModelEnum",
    "KnowledgeSplitConfig",
    "KnowledgeCreate",
    "Knowledge",
    "PageParams",
    "PageResponse",
    "RetrievalEnum",
    "RetrievalBySpaceRequest",
    "RetrievalByKnowledgeRequest",
    "RetrievalChunk",
    "Task",
    "TaskStatus",
    "Tenant",
]



from typing import Any, Dict, Generic, List, Optional, TypeVar

from pydantic import BaseModel, Field

T = TypeVar("T")


class OrCondition:
    field: str
    operator: str  # eq, neq, gt, gte, lt, lte, like, ilike etc.
    value: Any


class BasePageParams(BaseModel):
    page: int = Field(default=1, ge=1, description="page number")
    page_size: int = Field(default=10, ge=1, le=100, description="page size")


class PageParams(BaseModel):
    page: int = Field(default=1, ge=1, description="page number")
    page_size: int = Field(default=10, ge=1, le=100, description="page size")
    order_by: Optional[str] = Field(default=None, description="order by field")
    order_direction: Optional[str] = Field(default="asc", description="asc or desc")
    eq_conditions: Optional[Dict[str, Any]] = Field(
        default=None,
        description="list of equality conditions, each as a dict with key and value",
    )

    @property
    def offset(self) -> int:
        return (self.page - 1) * self.page_size

    @property
    def limit(self) -> int:
        return self.page_size


class PageResponse(BaseModel, Generic[T]):
    items: List[T]
    total: int
    page: int
    page_size: int
    total_pages: int



import importlib
import os
from enum import Enum
from pathlib import Path
from typing import Callable, Dict, Literal, Union, overload

from whiskerrag_types.interface.embed_interface import BaseEmbedding
from whiskerrag_types.interface.loader_interface import BaseLoader
from whiskerrag_types.interface.retriever_interface import BaseRetriever
from whiskerrag_types.model.knowledge import (
    EmbeddingModelEnum,
    KnowledgeSourceEnum,
    KnowledgeTypeEnum,
)
from whiskerrag_types.model.retrieval import RetrievalEnum

RegisterKeyType = Union[
    KnowledgeSourceEnum, KnowledgeTypeEnum, EmbeddingModelEnum, RetrievalEnum
]
RegisterInstanceType = Union[BaseLoader, BaseEmbedding, BaseRetriever]
RegisterDict = Dict[RegisterKeyType, RegisterInstanceType]


class RegisterTypeEnum(str, Enum):
    EMBEDDING = "embedding"
    KNOWLEDGE_LOADER = "knowledge_loader"
    RETRIEVER = "retriever"


_registry: Dict[RegisterTypeEnum, RegisterDict] = {}
_loaded_packages = set()


def register(
    register_type: RegisterTypeEnum,
    register_key: RegisterKeyType,
) -> Callable[[type], type]:
    def decorator(cls: type) -> type:
        setattr(cls, "_is_register_item", True)
        _registry.setdefault(register_type, {})
        expected_base: type
        if register_type == RegisterTypeEnum.EMBEDDING:
            expected_base = BaseEmbedding
        elif register_type == RegisterTypeEnum.KNOWLEDGE_LOADER:
            expected_base = BaseLoader
        elif register_type == RegisterTypeEnum.RETRIEVER:
            expected_base = BaseRetriever
        else:
            raise ValueError(f"Unknown register type: {register_type}")

        if not issubclass(cls, expected_base):
            raise TypeError(
                f"Class {cls.__name__} must inherit from {expected_base.__name__}"
            )
        setattr(cls, "_register_type", register_type)
        setattr(cls, "_register_key", register_key)
        print(f"Registering {cls.__name__} as {register_type} with key {register_key}")
        _registry[register_type][register_key] = cls  # type: ignore
        return cls

    return decorator


def init_register(package_name: str = "whiskerrag_utils") -> None:
    if package_name in _loaded_packages:
        return
    try:
        package = importlib.import_module(package_name)
        if package.__file__ is None:
            raise ValueError(
                f"Package {package_name} does not have a __file__ attribute"
            )
        package_path = Path(package.__file__).parent
        current_file = Path(__file__).name
        for root, _, files in os.walk(package_path):
            for file in files:
                if file == current_file or not file.endswith(".py"):
                    continue
                module_name = (
                    Path(root, file)
                    .relative_to(package_path)
                    .with_suffix("")
                    .as_posix()
                    .replace("/", ".")
                )

                if module_name == "__init__":
                    continue
                try:
                    importlib.import_module(f"{package_name}.{module_name}")
                except ImportError as e:
                    print(f"Error importing module {module_name}: {e}")
        _loaded_packages.add(package_name)

    except ImportError as e:
        print(f"Error importing package {package_name}: {e}")


@overload
def get_register(
    register_type: Literal[RegisterTypeEnum.KNOWLEDGE_LOADER],
    register_key: KnowledgeSourceEnum,
) -> BaseLoader: ...


@overload
def get_register(
    register_type: Literal[RegisterTypeEnum.EMBEDDING], register_key: EmbeddingModelEnum
) -> BaseEmbedding: ...


@overload
def get_register(
    register_type: Literal[RegisterTypeEnum.RETRIEVER], register_key: RetrievalEnum
) -> BaseRetriever: ...


def get_register(
    register_type: RegisterTypeEnum, register_key: RegisterKeyType
) -> RegisterInstanceType:
    register_instance = _registry.get(register_type, {}).get(register_key)
    if register_instance is None:
        raise KeyError(f"No loader registered for type: {register_type}.{register_key}")
    return register_instance



from abc import ABC, abstractmethod
from typing import List

from langchain_core.documents import Document

from whiskerrag_types.model.chunk import Chunk
from whiskerrag_types.model.knowledge import Knowledge


class BaseEmbedding(ABC):
    @abstractmethod
    async def embed_documents(
        self, knowledge: Knowledge, documents: List[Document]
    ) -> List[Chunk]:
        """
        Embed a list of documents.
        """
        pass

    @abstractmethod
    async def embed_text(self, text: str) -> List[float]:
        """
        Embed a text.
        """
        pass



from abc import ABC, abstractmethod
from typing import List

from ..interface import DBPluginInterface, LoggerManagerInterface, SettingsInterface
from ..model import Knowledge, KnowledgeCreate, Task, Tenant


class TaskEnginPluginInterface(ABC):
    settings: SettingsInterface
    logger: LoggerManagerInterface

    def __init__(
        self,
        logger: LoggerManagerInterface,
        settings: SettingsInterface,
    ):
        try:
            logger.info("TaskEngine plugin is initializing...")
            self.settings = settings
            self.logger = logger
            self.init()
            logger.info("TaskEngine plugin is initialized")
        except Exception as e:
            logger.error(f"TaskEngine plugin init error: {e}")

    @abstractmethod
    def init(self) -> None:
        """
        Initialize the task engine plugin, such as loading middleware, establishing contact with the task execution engine, etc.
        """
        pass

    @abstractmethod
    async def gen_knowledge_list(
        self, user_input: List[KnowledgeCreate], tenant: Tenant
    ) -> List[Knowledge]:
        """
        Generate a list of tasks from the user input.
        """
        pass

    @abstractmethod
    async def init_task_from_knowledge(
        self, knowledge_list: List[Knowledge], tenant: Tenant
    ) -> List[Task]:
        """
        Initialize a list of tasks from the knowledge list.
        """
        pass

    @abstractmethod
    async def execute_task(self, task_id: str) -> List[Task]:
        """
        Execute a task.
        """
        pass

    @abstractmethod
    async def batch_execute_task(
        self, task_list: List[Task], knowledge_list: List[Knowledge]
    ) -> List[Task]:
        """
        Execute a list of tasks.
        """
        pass

    @abstractmethod
    async def on_task_execute(self, db: DBPluginInterface) -> None:
        """
        Listen to the task execution status with no parameter restrictions.
        """
        pass

    @abstractmethod
    async def stop_on_task_execute(self) -> None:
        """
        Stop listening to the task execution status.
        """
        pass



import pathlib

from setuptools import find_namespace_packages, setup

# 获取当前目录
HERE = pathlib.Path(__file__).parent

# 读取 README.md 文件
README = (HERE / "README.md").read_text(encoding="utf-8")

setup(
    name="whiskerrag",
    version="0.0.1",
    description="A utility package for RAG operations",
    long_description=README,
    long_description_content_type="text/markdown",
    author="petercat.ai",
    author_email="antd.antgroup@gmail.com",
    packages=find_namespace_packages(where="src"),
    package_dir={"": "src"},
    python_requires=">=3.8.1,<4.0",
    setup_requires=[
        "wheel",
        "setuptools>=42.0.0",
    ],
    install_requires=[
        "pydantic>=2.0.0,<3.0.0",
        "PyGithub==2.3.0",
        "langchain-community>=0.2.11",
        "langchain-openai>=0.1.20",
        "langchain-core>=0.2.28",
        "langchain>=0.2.12",
        "requests>=2.32.3",
        "typing-extensions>=4.12.2",
    ],
    url="https://github.com/petercat-ai/whiskerrag_toolkit",
    project_urls={
        "Bug Tracker": "https://github.com/petercat-ai/whiskerrag_toolkit/issues",
        "Documentation": "https://github.com/petercat-ai/whiskerrag_toolkit#README.md",
        "Source Code": "https://github.com/petercat-ai/whiskerrag_toolkit",
    },
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
    ],
)



from enum import Enum
from typing import List

from pydantic import BaseModel, Field

from whiskerrag_types.model.chunk import Chunk
from whiskerrag_types.model.knowledge import EmbeddingModelEnum


class RetrievalEnum(str, Enum):
    SIMPLE = "simple"
    SIMILARITY = "similarity"


class RetrievalRequestBase(BaseModel):
    question: str = Field(..., description="The query question")
    embedding_model_name: EmbeddingModelEnum = Field(
        ..., description="The name of the embedding model"
    )
    similarity_threshold: float = Field(
        0.5,
        ge=0.0,
        le=1.0,
        description="The similarity threshold, ranging from 0.0 to 1.0.",
    )
    top: int = Field(1024, ge=1, description="The maximum number of results to return.")
    metadata_filter: dict = Field({}, description="metadata filter")
    # aggs: bool = Field(True, description="是否进行聚合")
    # rerank_mdl: Optional[str] = Field(None, description="重排序模型名称")
    # highlight: bool = Field(False, description="是否高亮显示")


class RetrievalBySpaceRequest(RetrievalRequestBase):
    space_id_list: List[str] = Field(..., description="space id list")


class RetrievalByKnowledgeRequest(RetrievalRequestBase):
    knowledge_id_list: List[str] = Field(..., description="knowledge id list")


class RetrievalChunk(Chunk):
    similarity: float = Field(..., description="The similarity of the chunk")
